
/*****************************************************/
/* wave.c                                            */
/* -- Simple WAVE generation.                        */
/*****************************************************/

#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <math.h>
#include <limits.h>
#include "snd/wave.h"

#define NTIMES 40

/* Types. */
typedef struct
	{
	DWORD lwTag; /* Tag. */
	DWORD lcb;   /* Size of data. */
	} CNK;       /* ChuNK. */

typedef struct
	{
	CNK   cnkRiff;     /* 'RIFF' chunk. */
	DWORD lwWaveId;    /* Must be 'WAVE'. */
	CNK   cnkFormat;   /* Wave specific format chunk. */
	PCMWAVEFORMAT wft; /* PCM stuff. */
	CNK   cnkData;     /* PCM Data chunk. */
	BYTE  rgb[FREQ];  /* The sound data. */
	} SND;             /* SouND. */

/* Constants. */
#define rFreqMin 20.f    /* Allowable frequency range. */
#define rFreqMax 25000.f
#define cbBufMax (0x00010000 - sizeof(SND))

/* Globals. */
static SND far *lpsnd;
static float    r2Pi;
static inited =0;
static sound_on = 0;

BOOL FInitSnd(void)
/*****************************************************/
/* -- Initialize this module.                        */
/*****************************************************/
	{
	short i;

	r2Pi = (float)(asin(1) * 4.0);

	if (NULL == (lpsnd = (SND far *)GlobalAllocPtr(GMEM_MOVEABLE |
	  GMEM_SHARE, sizeof(SND) )))
		return FALSE;

	lpsnd->cnkRiff.lwTag = *(long *)"RIFF";
	lpsnd->cnkRiff.lcb = sizeof(SND) - sizeof(CNK) - sizeof(BYTE);
	lpsnd->lwWaveId = *(long *)"WAVE";
	lpsnd->cnkFormat.lwTag = *(long *)"fmt ";
	lpsnd->cnkFormat.lcb = sizeof(PCMWAVEFORMAT);
	lpsnd->wft.wf.wFormatTag = WAVE_FORMAT_PCM;
	lpsnd->cnkData.lwTag = *(long *)"data";
	lpsnd->cnkData.lcb = 0;
        for(i = 0 ; i < FREQ ; i++ )
           lpsnd->rgb[i] = 0xFF;
	inited++;

        FPlaySnd((float)FREQ, -1, FREQ, 8, 1);
	return TRUE;
	}

void CloseSnd(void)
/*****************************************************/
/* -- Close this module.                             */
/*****************************************************/
	{
	if (lpsnd != (SND far *)NULL)
           GlobalFreePtr(lpsnd);
	}

void FDelay(long cms)
{
   DWORD mscStart;           /* Tone onset time. */

   if( cms > 0 ) {
      mscStart = timeGetTime();
      while (timeGetTime() - mscStart < (DWORD)cms)
			;
   }
}



BOOL FPlaySnd(float rFreq, long cms, UINT csps,
  UINT cbit, UINT cchn)
/*****************************************************/
/* -- Play the given sound.                          */
/* -- rFreq : Frequency of sound (Hz).               */
/* -- cms   : Duration (ms).                         */
/* -- csps  : Sampling rate (Hz).                    */
/* -- cbit  : Sample size (bits/sample/channel).     */
/* -- cchn  : Number of channels.                    */
/*****************************************************/
	{
	float r;         /* General purpose. */
	float csmpCycle; /* Samples per tone cycle. */
	UINT  cbSample;  /* Bytes/sample (per channel). */
	DWORD cbBufReq;  /* Required buffer size. */
	UINT  csmpBuf;   /* Sample count in buffer. */
	UINT  ismp, ismpT; /* Sample indices. */
	UINT  ichn;      /* Channel index. */
	DWORD mscStart;  /* Tone onset time. */

	if (rFreq < rFreqMin || rFreq > rFreqMax)
		return FALSE; /* Out of range. */

	csmpCycle = csps / rFreq;
		  csmpBuf = (UINT)(csmpCycle + 0.5) * FREQ/(50/NTIMES);
	cbSample = cbit / 8;
	cbBufReq = csmpBuf * cbSample * cchn;
	if (cbBufMax < cbBufReq)
		return FALSE; /* Too big. */

	lpsnd->wft.wf.nChannels = cchn;
	lpsnd->wft.wf.nSamplesPerSec = csps;
	lpsnd->wft.wf.nAvgBytesPerSec =
	  csps * cchn * cbSample;
	lpsnd->wft.wf.nBlockAlign = cchn * cbSample;
	lpsnd->wft.wBitsPerSample = cbit;
	lpsnd->cnkData.lcb = cbBufReq;
	lpsnd->cnkRiff.lcb = cbBufReq +
	  sizeof(SND) - sizeof(CNK) - sizeof(BYTE);

	return TRUE;
	}

UINT pos=0;
UINT pass_number =  0;
extern long clock_ticks;

fill_buf(int val)
{
	UINT i;
	UINT current_val = (UINT)( (clock_ticks/70000.0)*(FREQ/50.0))
                         + pass_number*(FREQ/50);

	if ((pos < current_val) && (current_val < (UINT)((FREQ/50.0)
           * NTIMES)))
	{
            for ( ; pos < (current_val-1) ; pos++)
               lpsnd->rgb[pos] = 0xFF;
             lpsnd->rgb[pos++] = val?0:0xFF;
             //lastvalue = /*val */ 0;
	}
}

CleanSndBuf()
{
        UINT limit = (UINT)((pass_number+1.0)*(FREQ/50.0))+1;

        if(inited && sound_on)
        {
          while(pos <= limit)
             lpsnd->rgb[pos++] = 0xFF;
          if(++pass_number == NTIMES)
          {
	     pos = 0;
             pass_number = 0;
          }
        }
}

void StopSnd(void)
/*****************************************************/
/* -- Stop playing the current sound.                */
/*****************************************************/
	{
        if(inited)
	   sndPlaySound(NULL, SND_ASYNC);
        sound_on = 0;
	}

void StartSnd(void)
/*****************************************************/
/* -- Stop playing the current sound.                */
/*****************************************************/
	{
        if(inited)
        {
	   sndPlaySound((LPCSTR)lpsnd,
	     SND_ASYNC | SND_MEMORY | SND_LOOP);
	  sound_on = 1;
      }
	}


/* EOF: Wave.c */
